#!/bin/sh

# ===
#
# Pi3Web UNIX configuration utility
#
# ===

echo ""
echo "Pi3Web setup and configuration Utility."
echo ""

#
# Globals
#
PI3_CONFIG="./bin/Config.pi3";
_PI3_HOSTNAME=`uname -n`
_PI3_PORT=80
_PI3_ADMINISTRATOR=""
_PI3_SEND=45
_PI3_RECV=60
_PI3_NUMPROC=10
_PI3_NUMTHREADS=5
_PI3_TYPENUMBER=1

# ---
#
# Get configuration values
#
# ---
get_values () {
    # 
    # Server Type
    #
    P_T=$_PI3_TYPENUMBER
	while [ 1 ] 
	do
        echo "# ---"
        echo "# "
        echo "# This server is packaged with a number of typically configurations."
        echo "# "
        echo "# Please select a configuration from the following list"
        echo "#    "
        echo "#  1 ... Features     [Intranet only - highlights features/tuitorial]"
        echo "#  2 ... Personal     [Small background web server on desktop platform]"
        echo "#  3 ... Internet     [Optimized for higher volumes, simpler features only]"
        echo "#  4 ... Proxy        [Caching proxy HTTP Server]"
        echo "#  5 ... Development  [CGI, WinCGI, PiAPI, FastCGI and debugging options]"
        echo "#    "
        echo "# ---"
        /bin/echo "Enter option (1-5) [$_PI3_TYPENUMBER]: \c"
        read P_T
        if [ -z "$P_T" ] ; then P_T=$_PI3_TYPENUMBER ; fi 
		if [ "$P_T" = 1 -o "$P_T" = 2 -o "$P_T" = 3 -o "$P_T" = 4 -o "$P_T" = 5 ] ;
		then
			break ;
		else
			echo "Invalid options $P_T";
		fi
    done 
    echo ""
    _PI3_TYPENUMBER=$P_T
    case $P_T in
        1)
        PI3_TYPE="Features.pi3"
        ;;
        2)
        PI3_TYPE="Personal.pi3"
        ;;
        3)
        PI3_TYPE="Internet.pi3"
        ;;
        4)
        PI3_TYPE="Proxy.pi3"
        ;;
        5)
        PI3_TYPE="Devel.pi3"
        ;;
        esac

    #
    # Hostname
    #
    echo "# ---"
    echo "# "
    echo "# A hostname must be specified for the Pi3Web HTTP server. The hostname"
    echo "# will be the used by the server to identify itself in URLs generated"
    echo "# for redirects and will also be used to lookup the IP address that this"
    echo "# server will listen on."
    echo "# "
    echo "# ---"
    /bin/echo "Enter hostname [$_PI3_HOSTNAME]: \c"
    read PI3_HOSTNAME
    if [ -z "$PI3_HOSTNAME" ] ; then PI3_HOSTNAME="$_PI3_HOSTNAME" ; fi
    echo ""
    _PI3_HOSTNAME=$PI3_HOSTNAME
    
    #
    # Port
    #
    echo "# ---"
    echo "#"
    echo "# A TCP port must be specified for this HTTP server. The TCP port"
    echo "# is the port that the HTTP will listen for requests on. This is"
    echo "# typically 80 for HTTP servers. "
    echo "# "
    echo "# Note that UNIX operating systems normally require that the server"
    echo "# initially run as user 'root' to bind on ports between 0 and 1024"
    echo "# "
    echo "# ---"
    /bin/echo "Enter port [$_PI3_PORT]: \c"
    read PI3_PORT
    if [ -z "$PI3_PORT" ] ; then PI3_PORT="$_PI3_PORT" ; fi
    echo ""
    _PI3_PORT=$PI3_PORT
    
    #
    # Administrator
    #
    echo "# ---"
    echo "#"
    echo "# E-mail address for the administrator for the primary virtual host."
    echo "# Used on messages and the like."
    echo "# "
    echo "# ---"
    if [ -z "$_PI3_ADMINISTRATOR" ] ; then _PI3_ADMINISTRATOR="webmaster@$_PI3_HOSTNAME" ; fi 
    /bin/echo "Enter administrators e-mail address [$_PI3_ADMINISTRATOR]: \c"
    read PI3_ADMINISTRATOR
    if [ -z "$PI3_ADMINISTRATOR" ] ; then PI3_ADMINISTRATOR="$_PI3_ADMINISTRATOR" ; fi
    echo ""
    _PI3_ADMINISTRATOR=$PI3_ADMINISTRATOR
    
    #
    # Send
    #
    echo "# ---"
    echo "# "
    echo "# Specify timeout for sending data. -1 specifies no timeout." 
    echo "# "
    echo "# ---"
    /bin/echo "Enter send timeout [$_PI3_SEND]: \c"
    read PI3_SEND
    if [ -z "$PI3_SEND" ] ; then PI3_SEND="$_PI3_SEND" ; fi
    echo ""
    _PI3_SEND=$PI3_SEND
    
    #
    # Recv
    #
    echo "# ---"
    echo "# "
    echo "# Specify timeout for receiving data. -1 specifies no timeout." 
    echo "# "
    echo "# ---"
    /bin/echo "Enter receive timeout [$_PI3_RECV]: \c"
    read PI3_RECV
    if [ -z "$PI3_RECV" ] ; then PI3_RECV="$_PI3_RECV" ; fi
    echo ""
    _PI3_RECV=$PI3_RECV
    
    #
    # Number of processes
    #
    echo "# ---"
    echo "# "
    echo "# Specify the number of processes this server will use."
    echo "# "
    echo "# ---"
    /bin/echo "Enter number of processes to use [$_PI3_NUMPROC]: \c"
    read PI3_NUMPROC
    if [ -z "$PI3_NUMPROC" ] ; then PI3_NUMPROC="$_PI3_NUMPROC" ; fi
    echo ""
    _PI3_NUMPROC=$PI3_NUMPROC
    
    #
    # Number of threads
    #
    echo "# ---"
    echo "# "
    echo "# Specify the number of threads this server will use."
    echo "# "
    echo "# ---"
    /bin/echo "Enter number of threads to use [$_PI3_NUMTHREADS]: \c"
    read PI3_NUMTHREADS
    if [ -z "$PI3_NUMTHREADS" ] ; then PI3_NUMTHREADS="$_PI3_NUMTHREADS" ; fi
    echo ""
    _PI3_NUMTHREADS=$PI3_NUMTHREADS ;
 
} ;

# ---
#
# function to confirm choices
#
# ---
confirm_config () {
    echo "# ---"
    echo "#"
    echo "# Review the configuration values you entered"
    echo "#"
    echo "# ---"
    echo "Server type file: $PI3_TYPE"
    echo "Hostname: $PI3_HOSTNAME"
    echo "Port: $PI3_PORT"
    echo "Administrator: $PI3_ADMINISTRATOR"
    echo "Receive timeout: $PI3_RECV"
    echo "Send timeout: $PI3_SEND"
    echo "Number of processes: $PI3_NUMPROC"
    echo "Number of threads: $PI3_NUMTHREADS"
}

# ---
#
# function to write the top level configuration file
#
# ---
write_config () {
    cat << 'EOF' > $PI3_CONFIG
#!Pi3

# ===
#
# Top level configuration file generated by Config.sh
#
# ===

# Include  standard IO and server stuff
include IO.cnf
include Server.cnf
include Pi3API.cnf

# Include the .pi3 file corresponding to the type of server being run
EOF
    echo "include $PI3_TYPE" >> $PI3_CONFIG
    cat << 'EOF' >> $PI3_CONFIG

# ---
#
# IO Object for main listening socket.
#
# ---
<Object>
    Name ServerIOObject
    Class TCPIPIOClass
    Type Passive

# A hostname must be specified for the Pi3Web HTTP server. The hostname
# will be the used by the server to identify itself in URLs generated
# for redirects and will also be used to lookup the IP address that this
# server will listen on.
EOF
    echo "    BindHost $PI3_HOSTNAME" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG

# A TCP port must be specified for this HTTP server. The TCP port
# is the port that the HTTP will listen for requests on. This is
# typically 80 for HTTP servers. 
#
# Note that UNIX operating systems normally require that the server
# initially run as user 'root' to bind on ports between 0 and 1024
EOF
    echo "    BindPort $PI3_PORT" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG

# Timeout accepting connections. -1 specifies no timeout.
    AcceptTimeout -1

# Specify timeout for sending data. -1 specifies no timeout. 
EOF
    echo "    RecvTimeout $PI3_RECV" >> $PI3_CONFIG;

    cat << 'EOF' >> $PI3_CONFIG

# Specify timeout for receiving data. -1 specifies no timeout. 
EOF
    echo "    SendTimeout $PI3_SEND" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG
    PeerIPVariable STRING:RemoteAddr
    PeerHostNameVariable STRING:RemoteHost
    LocalAddressVariable STRING:LocalAddr

# Flags
#
#   OwnDB - Object adds its own DB, rather than use the parents
#               this is necessary to avoid collision of values placed in
#               the DB between different connection. 
#
#   SetPeerAddress - IP address of remote peer with be put
#               into the specified variable.
#
#   DNSReverseLookup - Use reverse DNS to get the hostname of the 
#               peer and place it in the specified variable.
#
#   SetLocalAddress - Put local address in the specified variable.
    Flags OwnDB|SetPeerAddress|DNSReverseLookup|SetLocalAddress

# Pointer to file with lockfile name, to avoid collisions on accept
# with multiple processes
    AcceptLockFile  "../Logs/lockfile.txt"
</Object>

# ---
#
# Main virtual host object.
#
# ---
<Object>
    Name MainVirtualHostInformation
    Class VirtualHostClass
EOF
    cat << 'EOF' >> $PI3_CONFIG

# Administrator for the primary virtual host
EOF
    echo "    Administrator $PI3_ADMINISTRATOR" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG

# Hostname that this server uses in redirects.
EOF
    echo "    HostName $PI3_HOSTNAME" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG

# Port that the server uses in redirects.
EOF
    echo "    ServerPort $PI3_PORT" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG
</Object>

# ---
#
#    Thread dispatch object
#
#   This is an instance of MultiThreadedIOServerClass with appropriate
#   parameters
#
# ---
<Object>
    Name ThreadPoolDispatcher
    Class MultiThreadedIOServerClass

# The IOObject is the object that will be used for the listening socket
# it can be a real IOObject (such as TCP/IP), as test IO object which 
# feeds stub data into the server or a filter IO object which encrpts
# or logs data passed through it.
#
#    Example test IO object:
#   IOObject TestIOObject
#
#    Log IO to file:
#   IOObject SnoopIO IOObject="ServerIOObject" OutFile="./snoop.dbg"
#
    IOObject ServerIOObject

# Dispatch accepted IO connection to the HTTPLogicObject
    LogicObject HTTPLogicObject

# Number of requests to exit the child process after. This is probably
# not necessary (-1 disables it) and it applies to UNIX only
# (NT doesn't use it). 100 is a low number, 10000 is high. Put it to a
# low number if you suspect memory leaks. 1000 is a good number
	ExitAfter 1000

# Number of threads to use 
EOF
    echo "    MaxThreads $PI3_NUMTHREADS" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG
</Object>

# ---
#
# Main Pi3 object. 
#
# This is an instance of the UNIXDaemonClass to fork the server into
# one monitor process and 'NumberOfProcesses' child processes. 
#
# The monitor process automatically starts a new child processes whenever
# a child process exits.
#
# ---
<Object>
    Name Main
    Class UNIXDaemonClass

# Actual class to accept new IO requests and dispatch them
    ServerObject ThreadPoolDispatcher
    PIDFile "../Logs/pid.log"
    RLimitCpu Maximum

# User that child processes will run as. Server must run as root to 
# be able to change to the specified user. A user id number can be
# specified with #number. Server which start with root privileges should
# change to a user id with lesser privileges for security reasons.
#   User nobody

# Group that child processes will run as. Server must run as root to
# be able to change to a different group. A group id number can be
# specified with #number.
#   Group #-1

# generate a server unique lockfile name and write it into
# Logs/lockfile.txt
    LockFileName "../Logs/lockfile.txt"

# Message to write just before the server is ready to start accepting
# connections.
# default
    Message "Pi3Web HTTP Server Started"

# Specify the number of processes. 
EOF
    echo "    NumberOfProcesses $PI3_NUMPROC" >> $PI3_CONFIG;
    cat << 'EOF' >> $PI3_CONFIG

</Object>

EOF

# make configuration file executable (interpreter file)
	chmod +x $PI3_CONFIG
}

# ---
#
# Main loop
# 
# ---
    get_values
    ANS="n"
    while [ "$ANS" != y ] ; do
        confirm_config
        echo "";
        ANS=""
		while [ "$ANS" != "y" -a "$ANS" != "Y" -a "$ANS" != "N" -a "$ANS" != "n" ] ;
		do
    	    /bin/echo "Accept this configuration [Y/N]: \c";
     		read ANS
		done
	    if [ "$ANS" = "Y" -o "$ANS" = "y" ] ; then break ; fi
        get_values
    done
    write_config
	chmod +x $PI3_CONFIG

