<HTML><TITLE>BscAuth.c</TITLE><BODY><PRE>/*____________________________________________________________________________*\

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: BscAuth.c$
 * $Date: Sun Aug 10 06:37:31 1997$
 *
 Description:
	Basic authentication module.

\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include &ltassert.h>
#include &ltstdio.h>

#include "Pi3API.h"

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
#define KEY_CONF_REALM		"Realm"
#define BASIC_AUTH_TYPE		"Basic"

/*____________________________________________________________________________*\
 *
 Description:
	Documentation
\*____________________________________________________________________________*/
#if 0
	/*
	** HTML documentation for this handler
	*/
/*___+++HTMLDOC_BEGIN+++___*/
Name:
	<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A>

Description:
	Basic authentication is not included in this distribution.

Options:
&ltH5>Overview&lt/H5>
&ltTABLE BORDER=1>
&ltTH>Option
&ltTH>Default
&ltTH>Values
&ltTH>Short Description
&ltTH>Example(s)

&ltTR>
&ltTD>Realm
&ltTD>+
&ltTD>A string pattern
&ltTD>Realm to authenticate for
&ltTD>Realm="admin"

&lt/TABLE>
&ltSTRONG>-&lt/STRONG> in the &ltIT>default&lt/IT> indicates no default&ltBR>
&ltSTRONG>+&lt/STRONG> in the &ltIT>default&lt/IT> indicates the field is mandatory&ltBR>

&ltH4>Description of Options&lt/H4>
&ltH5>
	Realm
&lt/H5>
The realm to which this authentication applies. If the realm associated with
the request is different. This authentication module will pass up
handling of the request by returning PIAPI_CONTINUE.

Phase:
	CHECKAUTH

Returns:
	PIAPI_COMPLETED if authentication passed. INT_REDIRECT if the
	status was set to 401 to challenge, PIAPI_CONTINUE if this 
	handler choose to take no action and PIAPI_ERROR if this handler
	was invoked for a phase other than CHECKAUTH.

Note:
Example:
	&ltPRE>
	&ltObject&gt
		Name <A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A>
		Class BasicAuthClass
	&lt/Object&gt

	&ltObject&gt
		...
		Handle <A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> 
		...
	&lt/Object&gt
	&lt/PRE>
/*___+++HTMLDOC_END+++___*/
#endif

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
struct _BasicAuth
{
	char *pRealm;
};
typedef struct _BasicAuth BasicAuth;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int BasicAuth_fnParameter( void *pData, const char *pVar, const char *pVal,
	const char *pWhere )
{
	PIObject *pObject = (PIObject *)pData;
	<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *pBasicAuth = (<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObject );
	(void)pBasicAuth;
	assert( pVar );
	assert( pVal );

	if ( !<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_stricmp">PIUtil_stricmp</A>( pVar, KEY_CONF_REALM ) )
		{
		pBasicAuth->pRealm = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_strdup">PIUtil_strdup</A>( pVal );
		}
	else
		{
		/* ---
		Configuration error
		--- */
		<A HREF="/pidocs/APISpec/Pi2API.html#PILog_addMessage">PILog_addMessage</A>(
			<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getDB">PIObject_getDB</A>( pObject ),
			<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getConfigurationDB">PIObject_getConfigurationDB</A>( pObject ),
			PILOG_ERROR, "BasicAuth: %sUnknown directive '%s'.", pWhere, pVar );
		return 0;
		};
	return 1;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int BasicAuth_init( <A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *pBasicAuth, PIObject *pObject, int iArgc,
	const char *ppArgv[] )
{
	int iRet = <A HREF="/pidocs/APISpec/Pi2API.html#PIObject_readParameters">PIObject_readParameters</A>( pObject, iArgc, ppArgv,
		BasicAuth_fnParameter, pObject );

	/* --- set defaults --- */
	if ( iRet )
		{
		if ( !pBasicAuth->pRealm )
			{ pBasicAuth->pRealm = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_strdup">PIUtil_strdup</A>( "" ); };
		};

	return iRet;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int BasicAuth_onClassLoad( PIClass_LoadAction eLoad, int i,
	const char *a[] )
{
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int BasicAuth_constructor( PIObject *pObj,
	int iArgc, const char *ppArgv[] )
{
	<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *pBasicAuth = <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_malloc">PIUtil_malloc</A>( sizeof( <A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> ) );
	<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_setUserData">PIObject_setUserData</A>( pObj, pBasicAuth );
	if ( !BasicAuth_init( pBasicAuth, pObj, iArgc, ppArgv ) )
		{
		return PIAPI_ERROR;
		};
	return PIAPI_COMPLETED;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int BasicAuth_copyConstructor( PIObject *o, int i, const char *a[] )
{
	return PIAPI_ERROR;
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int BasicAuth_execute( PIObject *pObj, int iArgc,
		const char *ppArgv[] )
{
	PIHTTP *pPIHTTP;
	<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *pBasicAuth;
	const char *pRealm;
	char *pAuth;
	int i;

	if ( iArgc&lt1 )
		{ return PIAPI_ERROR; };

	pPIHTTP = (PIHTTP *)*ppArgv;

	if ( pPIHTTP->ciPhase!=PH_CHECKAUTH )
		{ return PIAPI_ERROR; };

	pBasicAuth = (<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObj );

	/* ---
	Get the realm
	--- */
	pRealm = (const char *)<A HREF="/pidocs/APISpec/Pi2API.html#PIDB_lookup">PIDB_lookup</A>( pPIHTTP->pResponseDB, PIDBTYPE_STRING,
		KEY_INT_AUTHENTICATIONREALM, 0 );

	if ( !pRealm || strcmp( pRealm, pBasicAuth->pRealm ) )
		{
		/* ---
		Not for this handler to authenticate
		--- */
		return PIAPI_CONTINUE;
		};

	/* ---
	NOTE:
		The next authentication section is not implemented, all requests
	are denied
	--- */
	if ( 0 )
		{
		/* ---
		Authenticated
		--- */
		<A HREF="/pidocs/APISpec/Pi2API.html#PIDB_replace">PIDB_replace</A>( pPIHTTP->pResponseDB, PIDBTYPE_STRING, KEY_INT_AUTHTYPE,
			BASIC_AUTH_TYPE, 0 );
		return PIAPI_COMPLETED;
		};

	/* ---
	Challenge
	--- */
	<A HREF="/pidocs/APISpec/Pi2API.html#PIDB_replace">PIDB_replace</A>( pPIHTTP->pResponseDB, PIDBTYPE_STRING, KEY_INT_AUTHTYPE,
		BASIC_AUTH_TYPE, 0 );
	i = sizeof( "Basic realm=\"\"" ) + strlen( pBasicAuth->pRealm ) + 1;
	pAuth = <A HREF="/pidocs/APISpec/Pi3API.html#PIHTTP_allocMem">PIHTTP_allocMem</A>( pPIHTTP, i );
	sprintf( pAuth, "Basic realm=\"%s\"", pBasicAuth->pRealm );
	<A HREF="/pidocs/APISpec/Pi2API.html#PIDB_replace">PIDB_replace</A>( pPIHTTP->pResponseDB, PIDBTYPE_RFC822,
		KEY_HTTP_WWW_AUTHENTICATE, pAuth, 0 );

	/* ---
	Redirect because of authentication failure
	--- */
	return <A HREF="/pidocs/APISpec/Pi3API.html#HTTPUtil_doHTTPError">HTTPUtil_doHTTPError</A>( pPIHTTP, ST_UNAUTHORIZED );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
PUBLIC_PIAPI int BasicAuth_destructor( PIObject *pObj, int i, const char *a[] )
{
	<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *pAuth = (<A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A> *)<A HREF="/pidocs/APISpec/Pi2API.html#PIObject_getUserData">PIObject_getUserData</A>( pObj );
	if ( pAuth->pRealm )
		{ <A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pAuth->pRealm ); };
	<A HREF="/pidocs/APISpec/PiAPI.html#PIUtil_free">PIUtil_free</A>( pAuth );
	return PIAPI_COMPLETED;
}

#if 0
/*___+++CNF_BEGIN+++___*/
	&ltClass>
		Name BasicAuthClass
		Type LogicExtension
		Library Plugins
		OnClassLoad BasicAuth_onClassLoad
		Constructor BasicAuth_constructor
		CopyConstructor BasicAuth_copyConstructor
		Destructor BasicAuth_destructor
		Execute BasicAuth_execute
	&lt/Class>

	&ltObject>
		Name <A HREF="/pidocs/Objects/Plugins.html#BasicAuth">BasicAuth</A>
		Class BasicAuthClass
	&lt/Object>

/*___+++CNF_END+++___*/
#endif

</PRE></BODY></HTML>